<?php
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_2/includes/database.php $
 * $Id: database.php 1527 2009-04-06 20:33:04Z quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

// Supported database types:
// 'mysql' - MySQL 4.1 or greater
// 'pgsql' - PostgreSQL 8.1 or greater

function db_open ($type, $sock, $host, $port, $user, $pass, $name)
{
	if (!strlen($sock) && !strlen($host))
		die('Invalid database configuration - must specify either hostname or UNIX socket');
	if (strlen($sock) && strlen($host))
		die('Invalid database configuration - cannot specify both hostname and UNIX socket');

	// Select database driver and construct DSN for connection
	switch ($type)
	{
	case 'mysql':
		$dsn = 'mysql:';
		if ($sock)
			$dsn .= 'unix_socket='. $sock .';';
		elseif ($host)
		{
			$dsn .= 'host='. $host .';';
			if ($port)
				$dsn .= 'port='. $port .';';
		}
		$dsn .= 'dbname='. $name;
		break;
	case 'pgsql':
		$dsn = 'pgsql:';
		if ($sock)
			$dsn .= 'host='. $sock .' ';
		elseif ($host)
		{
			$dsn .= 'host='. $host .' ';
			if ($port)
				$dsn .= 'port='. $port .' ';
		}
		$dsn .= 'dbname='. $name;
		break;
/*	// The SQLite PDO driver in PHP 5.2 is broken - see http://bonsai.php.net/bug.php?id=45259 for details
	// The most visible effect of this bug is that the turns script terminates all sessions regardless of age
	case 'sqlite':
		if ($host)
			die('SQLite driver must specify filename in SOCK configuration value');
		$dsn = 'sqlite:'. $sock;
		break;
*/
	default:
		die('An unsupported database driver has been specified!');
	}

	// Attempt to open database connection
	try
	{
		$db = new PDO($dsn, $user, $pass);
	}
	catch (PDOException $e)
	{
		return NULL;
	}

	$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);

	// Database-specific post-connect actions
	switch ($type)
	{
	case 'mysql':
		$db->query("SET NAMES 'utf8'");
		$db->query("SET SESSION sql_mode=''");
		break;
	case 'sqlite':
		$db->query("PRAGMA encoding = 'UTF-8'");
		$db->query("PRAGMA locking_mode = EXCLUSIVE");
		$db->query("PRAGMA synchronouse = OFF");
		$db->sqliteCreateFunction('LEAST', 'sqlite_least', 2);
		break;
	}

	return $db;
}

// Shortcut functions for binding different types
// Integer
function db_bindIntValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_INT);
}
// String
function db_bindStrValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_STR);
}
// Float
function db_bindFltValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_STR);
}
// Automatically bind all parameters based on type (for cases where the parameter count varies)
function db_bindAllParms ($query, $parms)
{
	foreach ($parms as $i => $val)
	{
		if (is_numeric($val))
		{
			if ($val - floor($val) > 0.00001)
				db_bindFltValue($query, $i + 1, $val);
			else	db_bindIntValue($query, $i + 1, $val);
		}
		else	db_bindStrValue($query, $i + 1, $val);
	}
}
// Clear the contents of a table and reset its auto-incrementing ID
function db_clearTable ($db, $table)
{
	switch ($db->getAttribute(PDO::ATTR_DRIVER_NAME))
	{
	case 'mysql':
		if (!$db->query('TRUNCATE TABLE '. $table))
			return FALSE;
		break;
	case 'pgsql':
		if (!$db->query('TRUNCATE TABLE '. $table))
			return FALSE;
		$seq = db_getSequence($db, $table);
		if ($seq)
		{
			if (!$db->query('ALTER SEQUENCE '. $seq .' RESTART WITH 1'))
				return FALSE;
		}
		break;
	case 'sqlite':
		if (!$db->query('DELETE FROM '. $table))
			return FALSE;
		break;
	}
	return TRUE;
}
// Retrieves a database's sequence, if it exists
function db_getSequence ($db, $table)
{
	global $sequences;
	// MySQL and SQLite do not use sequences
	if (in_array($db->getAttribute(PDO::ATTR_DRIVER_NAME), array('mysql', 'sqlite')))
		return NULL;
	// Not all tables have sequences associated with them
	if (!isset($sequences[$table]))
		return NULL;
	return $sequences[$table];
}
// Modifies the query to limit rows returned
function db_setLimit ($db, $query, $rows, $offset = 0)
{
	switch ($db->getAttribute(PDO::ATTR_DRIVER_NAME))
	{
	case 'mysql':
		if ($offset)
			$query .= ' LIMIT '. $offset .','. $rows;
		else	$query .= ' LIMIT '. $rows;
		break;
	case 'pgsql':
	case 'sqlite':
		$query .= ' LIMIT '. $rows;
		if ($offset)
			$query .= ' OFFSET '. $offset;
		break;
	}
	return $query;
}

// evaluate an SQL query, return first cell of first row
// useful for "SELECT COUNT(*) ..." queries
function sqleval ($db, $query, $parms = NULL)
{
	$q = $db->prepare($query) or warning('SQL prepare failed', 1);
	if ($parms)
		db_bindAllParms($q, $parms);
	$q->execute() or warning('SQL execute failed', 1);
	return $q->fetchColumn();
}

// 1-liner for executing quick SQL queries, intended only for turns.php
function sqlquery ($db, $query, $parms = NULL)
{
	$q = $db->prepare($query) or warning('SQL prepare failed', 1);
	if ($parms)
		db_bindAllParms($q, $parms);
	$q->execute() or warning('SQL execute failed', 1);
	return $q;
}

// SQLite does not implement the LEAST() function (it uses MIN() instead), so it needs to be implemented here
function sqlite_least ($num1, $num2)
{
	return min($num1, $num2);
}
?>
